@@ -1,6 +1,6 @@
 from .recordingextractor import RecordingExtractor
 from .sortingextractor import SortingExtractor
-from .cacherecordingextractor import CacheRecordingExtractor
+from .cacheextractors import CacheRecordingExtractor, CacheSortingExtractor
 from .subsortingextractor import SubSortingExtractor
 from .subrecordingextractor import SubRecordingExtractor
 from .multirecordingchannelextractor import concatenate_recordings_by_channel, MultiRecordingChannelExtractor
@@ -10,7 +10,8 @@ from .multisortingextractor import concatenate_sortings, MultiSortingExtractor
 from .extractorlist import *
 
 from . import example_datasets
-from .extraction_tools import load_probe_file, save_to_probe_file, read_binary, write_to_binary_dat_format, \
-    get_sub_extractors_by_property, load_extractor_from_json, load_extractor_from_dict
+from .extraction_tools import load_probe_file, save_to_probe_file, read_binary, write_to_binary_dat_format,\
+    write_to_h5_dataset_format, get_sub_extractors_by_property, load_extractor_from_json, load_extractor_from_dict, \
+    load_extractor_from_pickle
 
 from .version import version as __version__
